@@ -18,7 +18,7 @@ This product includes software developed by and/or derived from the Access
 Grid Project (http://www.accessgrid.org) to which the U.S. Government retains 
 certain rights."""
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = '$Id: client.py 7772 2010-12-09 13:40:45Z pjkersha $'
+__revision__ = '$Id: client.py 7928 2011-08-12 13:16:26Z pjkersha $'
 import logging
 log = logging.getLogger(__name__)
 
@@ -320,6 +320,7 @@ class MyProxyClient(object):
     MYPROXY_SERVER_DN_ENVVARNAME = 'MYPROXY_SERVER_DN'
     
     GLOBUS_LOCATION_ENVVARNAME = 'GLOBUS_LOCATION'
+    X509_USER_PROXY_ENVVARNAME = 'X509_USER_PROXY'
     
     GET_CMD="""VERSION=MYPROXYv2
 COMMAND=0
@@ -368,8 +369,18 @@ TRUSTED_CERTS=1"""
     TRUSTED_CERTS_FIELDNAME = 'TRUSTED_CERTS'
     TRUSTED_CERTS_FILEDATA_FIELDNAME_PREFIX = 'FILEDATA_'
     
-    HOSTCERT_SUBDIRPATH = ('etc', 'hostcert.pem')
-    HOSTKEY_SUBDIRPATH = ('etc', 'hostkey.pem')
+    HOSTCERT_FILENAME = 'hostcert.pem'
+    HOSTKEY_FILENAME = 'hostkey.pem'
+    
+    GRID_SECURITY_DIRPATH = '/etc/grid-security'
+    
+    HOSTCERT_GRID_FILEPATH = os.path.join(GRID_SECURITY_DIRPATH, 
+                                          HOSTCERT_FILENAME)
+    HOSTKEY_GRID_FILEPATH = os.path.join(GRID_SECURITY_DIRPATH, 
+                                         HOSTKEY_FILENAME)
+    
+    HOSTCERT_SUBDIRPATH = ('etc', HOSTCERT_FILENAME)
+    HOSTKEY_SUBDIRPATH = ('etc', HOSTKEY_FILENAME)
     
     PROXY_FILE_PERMISSIONS = 0600
     
@@ -607,18 +618,18 @@ TRUSTED_CERTS=1"""
         return self.__proxyCertLifetime
     
     def _setProxyCertLifetime(self, val):
-        if isinstance(val, basestring):
+        if isinstance(val, (basestring, float)):
             self.__proxyCertLifetime = int(val)
         elif isinstance(val, int):
             self.__proxyCertLifetime = val
         else:
-            raise TypeError("Expecting int type for proxyCertLifetime "
-                            "attribute")
+            raise TypeError("Expecting int, float or string type for input "
+                            "proxyCertLifetime attribute")
     
     proxyCertLifetime = property(fget=_getProxyCertLifetime,
                                  fset=_setProxyCertLifetime,
-                                 doc="Default proxy cert. lifetime used in "
-                                     "logon request")
+                                 doc="Default proxy cert. lifetime (seconds) "
+                                     "used in logon request")
 
     def _getCACertDir(self):
         return self.__caCertDir
@@ -835,7 +846,53 @@ TRUSTED_CERTS=1"""
             dat = dat[ind + len + 4:]
            
         return pemCerts
-       
+
+    @classmethod
+    def locateClientCredentials(cls, enableTmpFileLoc=False):
+        """Find the location of a client certificate and private key to use to 
+        authenticate with the server based on the various default locations
+        that MyProxy/Globus support
+        
+        @param enableTmpFileLoc: enable setting based on /tmp/x509up_<uid>,
+        defaults to False
+        @type enableTmpFileLoc: bool
+        @return: private key and certificate file location to use based on the
+        current environment
+        @rtype: tuple
+        """
+        sslKeyFile = None
+        sslCertFile = None
+        
+        x509UserProxy = os.environ.get(cls.X509_USER_PROXY_ENVVARNAME)
+        if x509UserProxy is not None:
+            sslKeyFile = x509UserProxy
+            sslCertFile = x509UserProxy
+            
+        elif enableTmpFileLoc and os.access(cls.DEF_PROXY_FILEPATH, os.R_OK):
+            sslKeyFile = cls.DEF_PROXY_FILEPATH
+            sslCertFile = cls.DEF_PROXY_FILEPATH
+            
+        elif (os.access(cls.HOSTKEY_GRID_FILEPATH, os.R_OK) and 
+              os.access(cls.HOSTCERT_GRID_FILEPATH, os.R_OK)):
+            sslKeyFile = cls.HOSTKEY_GRID_FILEPATH
+            sslCertFile = cls.HOSTCERT_GRID_FILEPATH
+            
+        else:
+            globusLoc = os.environ.get(cls.GLOBUS_LOCATION_ENVVARNAME)
+            if globusLoc:
+                sslKeyFile = os.path.join(globusLoc, 
+                                          *cls.HOSTKEY_SUBDIRPATH)
+                if os.access(sslKeyFile, os.R_OK):
+                    sslCertFile = os.path.join(globusLoc, 
+                                           *cls.HOSTCERT_SUBDIRPATH)
+                else:
+                    # Access to the private key is prohibited default to
+                    # username/password based authentication             
+                    sslKeyFile = None
+                    sslCertFile = None
+                    
+        return sslKeyFile, sslCertFile
+                
     @classmethod
     def writeProxyFile(cls, proxyCert, proxyPriKey, userX509Cert, 
                        filePath=None):
@@ -1231,14 +1288,18 @@ TRUSTED_CERTS=1"""
         
     def logon(self, 
               username, 
-              passphrase, 
+              passphrase,
+              credname=None, 
               lifetime=None, 
               keyPair=None, 
               certReq=None, 
               nBitsForKey=PRIKEY_NBITS, 
               bootstrap=False,
               updateTrustRoots=False,
-              authnGetTrustRootsCall=False):
+              authnGetTrustRootsCall=False,
+              sslCertFile=None,
+              sslKeyFile=None,
+              sslKeyFilePassphrase=None):
         """Retrieve a proxy credential from a MyProxy server
         
         Exceptions:  MyProxyClientGetError, MyProxyClientRetrieveError
@@ -1250,6 +1311,10 @@ TRUSTED_CERTS=1"""
         @param passphrase: pass-phrase for private key of credential held on
         server
         
+        @type credname: string / None type
+        @param credname: optional credential name - provides additional means
+        to specify credential to be retrieved
+        
         @type lifetime: int
         @param lifetime: lifetime for generated certificate
         
@@ -1289,6 +1354,15 @@ TRUSTED_CERTS=1"""
         or "bootstrap" keywords are set.  This is not recommended for 
         bootstrap since in this case the server is NOT authenticated by this 
         client. 
+        
+        @param sslCertFile: applies to SSL client based authentication - 
+        alternative to username/pass-phrase based.  This certificate is used for 
+        authentication with MyProxy server over the SSL connection.  If not set,
+        this argument defaults to $GLOBUS_LOCATION/etc/hostcert.pem 
+        @param sslKeyFile: corresponding private key file.  See explanation
+        for sslCertFile
+        @param sslKeyFilePassphrase: passphrase for sslKeyFile.  Omit if the
+        private key is not password protected.  
         """
         if bootstrap:
             log.info('Bootstrapping MyProxy server root of trust.')
@@ -1310,8 +1384,28 @@ TRUSTED_CERTS=1"""
             
         lifetime = lifetime or self.proxyCertLifetime
 
+        # Basic sanity check on username to avoid overhead on server
+        if not username:
+            raise MyProxyClientGetError('No username has been set')
+        
+        # Sanitise password - None is legal for modes where username/password
+        # based authentication is not required
+        if passphrase is None:
+            passphrase = ''
+        
+        # Check for credential name setting
+        if credname:
+            if not isinstance(credname, basestring):
+                raise TypeError('Expecting string type or None for "credname" '
+                                'input argument')
+                
+            # Make a concatenated username credential parameter to pass
+            userid = "%s-%s" % (username, credname)
+        else:
+            userid = username
+                     
         # Certificate request may be passed as an input but if not generate it
-        # here request here 
+        # here 
         if certReq is None:
             # If no key pair was passed, generate here
             if keyPair is None:
@@ -1321,16 +1415,25 @@ TRUSTED_CERTS=1"""
 
         if keyPair is not None: 
             pemKeyPair = crypto.dump_privatekey(crypto.FILETYPE_PEM, keyPair)
-        
+
+
+        # Check for certificate and private key set in environment which can
+        # be used to authenticate with.  This is an alternative to the username
+        # / passphrase based authentication
+        if sslKeyFile is None or sslCertFile is None:
+            sslKeyFile, sslCertFile = self.__class__.locateClientCredentials()
+                
         # Set-up SSL connection
-        conn = self._initConnection()
+        conn = self._initConnection(certFile=sslCertFile,
+                                    keyFile=sslKeyFile,
+                                    keyFilePassphrase=sslKeyFilePassphrase)
         conn.connect((self.hostname, self.port))
         
         # send globus compatibility stuff
         conn.write('0')
     
         # send get command - ensure conversion from unicode before writing
-        cmd = MyProxyClient.GET_CMD % (username, passphrase, lifetime)
+        cmd = MyProxyClient.GET_CMD % (userid, passphrase, lifetime)
             
         conn.write(str(cmd))
         
